# Generated by Django 4.0 on 2022-03-03 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('server', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='webpage',
            name='logout_all',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='webpage',
            name='response_type',
            field=models.CharField(choices=[('#', 'Hash'), ('?', 'Params')], default='#', max_length=1),
        ),
    ]
